module stored_variables
    implicit none
!
    integer :: KK
!   zdiag : 1d array of diagonal elements of the tridiagonal matrix
!   for Crank-Nicolson method
!   zleft : 1d array of the elements left to the diagonal ones
!   zright : 1d array of the elements right to the diagonal ones
    double precision, allocatable :: zdiag(:),zleft(:),zright(:)
end module stored_variables
!
program tridiagonal_solver
    use stored_variables
    implicit none

    integer :: i
    double precision :: eps
    double precision,allocatable :: input(:), output(:)

    KK = 5
    eps = 1d-3
    allocate(zdiag(0:KK),zleft(0:KK),zright(0:KK))
    allocate(input(0:KK),output(0:KK))

    zdiag(:) = -4.0d0
    zleft(:) = 1.0d0
    zright(:) = 1.0d0
    input(:) = 1.0d0
!
!   solve the matrix equation
    call tridiagonal(zleft,zdiag,zright,output,input,0,KK)

    write(*,'(6F12.8)') output(:)

    do i=0,5
        zdiag(i) = 1.0d0
        zleft(i) = -0.1d0*eps*i
        zright(i) = -0.2d0*eps*i
        input(i) = i*1.0d0
    end do
!
!   solve the matrix equation
    call tridiagonal(zleft,zdiag,zright,output,input,0,KK)

    write(*,'(6F12.8)') output(:)

end program tridiagonal_solver

!--------------------------------------------------------------------
! this subroutine solves CX=Y, where C is a tridiagonal matrix
subroutine tridiagonal(cu,cd,cl,x,y,n0,nm)
!
    implicit none
    integer, intent(in) :: n0, nm
    double precision, intent(in) :: cu(0:nm), cd(0:nm), cl(0:nm)
    double precision, intent(out) :: x(0:nm)
    double precision, intent(in) :: y(0:nm)
    double precision :: gam(0:nm), bet
    integer :: j
!
    x(0) = 0.0d0
!   if (cd(n0).eq.0.0d0) stop 'Zero matrix element'
    bet = cd(n0)
    x(n0) = y(n0)/bet
    do j=1,nm
        gam(j) = cl(j-1)/bet
        bet = cd(j)-cu(j)*gam(j)
!       if (bet.eq.0.0d0) stop 'Bad zero term in tridiagonal'
        x(j) = (y(j)-cu(j)*x(j-1))/bet
    end do
    do j=nm-1,n0,-1
        x(j) = x(j) - gam(j+1)*x(j+1)
    end do
end subroutine tridiagonal

